package java.lang;

import java.util.Iterator;
import java.util.Set;
import java.util.IdentityHashMap;

class ApplicationShutdownHooks
{
    private static IdentityHashMap<Thread, Thread> hooks;
    
    private ApplicationShutdownHooks() {
    }
    
    static synchronized void add(Thread thread) {
        if (ApplicationShutdownHooks.hooks == null) {
            throw new IllegalStateException("Shutdown in progress");
        }
        if (thread.isAlive()) {
            throw new IllegalArgumentException("Hook already running");
        }
        if (ApplicationShutdownHooks.hooks.containsKey(thread)) {
            throw new IllegalArgumentException("Hook previously registered");
        }
        ApplicationShutdownHooks.hooks.put(thread, thread);
        return;
    }
    
    static synchronized boolean remove(Thread key) {
        if (ApplicationShutdownHooks.hooks == null) {
            throw new IllegalStateException("Shutdown in progress");
        }
        if (key == null) {
            throw new NullPointerException();
        }
        return ApplicationShutdownHooks.hooks.remove(key) != null;
    }
    
    static void runHooks() {
        Set<Thread> keySet;
        synchronized (ApplicationShutdownHooks.class) {
            keySet = ApplicationShutdownHooks.hooks.keySet();
            ApplicationShutdownHooks.hooks = null;
        }
        Iterator<Object> iterator = keySet.iterator();
        while (iterator.hasNext()) {
            ((Thread)iterator.next()).start();
        }
        for (Thread thread : keySet) {
            Label_0085: {
                try {
                    thread.join();
                }
                catch (InterruptedException ex) {
                    continue;
                }
            }
        }
    }
    
    static {
        try {
            Shutdown.add(1, false, new Runnable() {
                @Override
                public void run() {
                    ApplicationShutdownHooks.runHooks();
                }
            });
            ApplicationShutdownHooks.hooks = new IdentityHashMap();
        }
        catch (IllegalStateException ex) {
            ApplicationShutdownHooks.hooks = null;
        }
    }
}
